/* $Id: LADSPAPluginDescriptorFactory.cc,v 1.3 2003/08/08 16:19:32 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "LADSPAPluginDescriptorFactory.hh"
#include "LADSPAPluginDescriptor.hh"
#include "ladspa.h"

#include "dev/SCC/SCCDescriptor.hh"
#include "dev/PSG/PSGDescriptor.hh"
#include "dev/CVFreq/CVFreqDescriptor.hh"
#include "dev/MUSIC/MUSICDescriptor.hh"
#include "dev/MUSICDrum/MUSICDrumDescriptor.hh"
#include "dev/SID/SIDDescriptor.hh"

std::map<unsigned long, LADSPAPluginDescriptor*> LADSPAPluginDescriptorFactory::m_descriptorMap;

void LADSPAPluginDescriptorFactory::init()
{
	m_descriptorMap[0] = new CVFreqDescriptor();
	m_descriptorMap[1] = new SCCDescriptor();
	m_descriptorMap[2] = new PSGDescriptor();
	m_descriptorMap[3] = new MUSICDescriptor();
	m_descriptorMap[4] = new MUSICDrumDescriptor();
	m_descriptorMap[5] = new SIDDescriptor();
}

void LADSPAPluginDescriptorFactory::fini()
{
}

LADSPAPluginDescriptor* LADSPAPluginDescriptorFactory::get(unsigned long index)
{
	std::map<unsigned long, LADSPAPluginDescriptor*>::const_iterator i = m_descriptorMap.find(index);
	if (i == m_descriptorMap.end())
	{
		return 0;
	}
	else
	{
		return i->second;
	}
}

const LADSPA_Descriptor* ladspa_descriptor(unsigned long Index)
{
	LADSPAPluginDescriptor* l = LADSPAPluginDescriptorFactory::get(Index);
	if (l != 0)
	{
		return l->getLADSPA_Descriptor();
	}
	return 0;
}

static class IniFini
{
public:
	IniFini()
	{
		LADSPAPluginDescriptorFactory::init();
	}
	~IniFini()
	{
		LADSPAPluginDescriptorFactory::fini();
	}
} ini_fini;
