/* $Id: SCCInstance.cc,v 1.3 2003/09/17 19:08:14 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

//
// a single channel of an SCC
//
// based on code of openMSX
//

#include "SCCInstance.hh"

#include <cassert>
#include <cstring>
#include <iostream>

SCCInstance::SCCInstance(unsigned long sampleRate)
:sampleRate(sampleRate),
 realstep((unsigned int)(((unsigned int)(1 << 31)) / sampleRate)),
 port_preset_backup(-1)
{
	reset();

#include "gen/presets.cc"

}

SCCInstance::~SCCInstance()
{
}

void SCCInstance::connect_port(unsigned long port, LADSPA_Data* dataLocation)
{
//	std::cerr << "SCCInstance " << load_number << " Connecting port " << port << " datalocation:" << dataLocation << "\n";
	switch (port)
	{
		case 0:
			port_freq   = dataLocation;
			break;
		case 1:
			port_amp    = dataLocation;
			break;
		case 2:
			port_preset  = dataLocation;
			break;
		case 3:
			port_output = dataLocation;
			break;
		default:
			assert(false);
//			assert(port < 32+5);
//			port_byte[port-5] = dataLocation;
			break;
	}
}

void SCCInstance::activate()
{
//	std::cerr << "SCCInstance activate\n";
}

void SCCInstance::run(unsigned long SampleCount)
{
//	for (int i=0; i < 32; i++)
//	{
//		wave[i] = *(port_byte[i]);
//	}

	if (*port_preset != port_preset_backup)
	{
		port_preset_backup = *port_preset;
		unsigned int index = static_cast<unsigned int>(port_preset_backup);
		std::cerr << "Changing to preset[" << index << "] " << presets[index].name << "\n";
		for (int i=0; i < 32; i++)
		{
			wave[i] = presets[index].shape[i];
		}
	}

	LADSPA_Data real_freq = *port_freq;

	freq = static_cast<unsigned int>((CLOCK_FREQ / (32 * (real_freq))) - 1);

	incr = (freq <= 8) ? 0 : (2 << GETA_BITS) / (freq + 1);

	LADSPA_Data mod_amp;
	mod_amp = ((*port_amp) <= 0.0010001)?0:(*port_amp);
	LADSPA_Data* buf = port_output;
	calcVolAdjustedWave();
	while (SampleCount--)
	{
		scctime += realstep;
		unsigned advance = scctime / SCC_STEP;
		scctime %= SCC_STEP;
		count += incr *advance;
		*buf = (((float)volAdjustedWave[(count >> GETA_BITS) & 0x1F])/127)*(mod_amp);
		++buf;
	}
}

void SCCInstance::reset()
{
	for (int j = 0; j < 32; j++)
	{
		wave[j] = 0;
		volAdjustedWave[j] = 0;
	}
	incr    = 0;
	count   = 0;
	freq    = 0;
	volume  = 15;
	scctime = 0;

// example sample, sinewave;
//	for (int i=0; i < 32; i++)
//	{
//		wave[i] = sin((2*3.141596/32)*i)*127;
//	}
/*
	wave[ 8] = 0x7F;
	wave[10] = 0x7F;
	wave[12] = 0x7F;
	wave[14] = 0x7F;
	wave[16] = 0x7F;
	wave[18] = 0x7F;
	wave[20] = 0x7F;
	wave[22] = 0x7F;
	wave[ 9] = 0x80;
	wave[11] = 0x80;
	wave[13] = 0x80;
	wave[15] = 0x80;
	wave[17] = 0x80;
	wave[19] = 0x80;
	wave[21] = 0x80;
	wave[23] = 0x80;
*/
}

void SCCInstance::calcVolAdjustedWave()
{
	for (int i=0; i<32; i++)
	{
		volAdjustedWave[i] = (((signed char)wave[i])*volume)/16;
	}
}
