/* $Id: SIDDescriptor.cc,v 1.3 2003/09/19 12:26:49 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "SIDDescriptor.hh"
#include "LADSPAPluginInstance.hh"
#include "LADSPAPortDescriptor.hh"
#include "LADSPAPluginDescriptor.hh"
#include "SIDInstance.hh"

#include <string>
#include <sstream>

LADSPAPortDescriptor* newBoolPortDescriptor(const std::string& name);

SIDDescriptor::SIDDescriptor()
:
	LADSPAPluginDescriptor
		(
			2066,
			std::string("SID"),
			std::string("SID chip"),
			std::string("Joost Yervante Damad"),
			std::string("GPLv2")
		)
{
	LADSPAPortDescriptor *port;

	port = new LADSPAPortDescriptor
		(
			std::string("Amplitude"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Control ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			15
		);
	addPort(port);

	port = new LADSPAPortDescriptor
		(
			std::string("Freq (Hz) 1"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::SampleRate   |
				LADSPAPortDescriptor::Logarithmic  |
				LADSPAPortDescriptor::Default440,

			0,
			0.5
		);
	addPort(port);
		port = new LADSPAPortDescriptor
		(
			std::string("Freq (Hz) 2"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::SampleRate   |
				LADSPAPortDescriptor::Logarithmic  |
				LADSPAPortDescriptor::Default440,

			0,
			0.5
		);
	addPort(port);
	port = new LADSPAPortDescriptor
		(
			std::string("Freq (Hz) 3"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::SampleRate   |
				LADSPAPortDescriptor::Logarithmic  |
				LADSPAPortDescriptor::Default440,

			0,
			0.5
		);
	addPort(port);

	addPort(newBoolPortDescriptor("MOS8580"));
	addPort(newBoolPortDescriptor("Filter On", true));
	addPort(newBoolPortDescriptor("Filter External On"));
	addPort(newBoolPortDescriptor("Filter LP"));
	addPort(newBoolPortDescriptor("Filter BP"));
	addPort(newBoolPortDescriptor("Filter HP"));
	addPort(newRangePortDescriptor("Filter Cutoff Freq", 0, 2047));
	addPort(newRangePortDescriptor("Filter Resonance", 0, 15));
	addPort(newBoolPortDescriptor("3 Off"));
	
	addPort(newBoolPortDescriptor("Noise 1"));
	addPort(newBoolPortDescriptor("Pulse 1"));
	addPort(newRangePortDescriptor("Pulse Width 1", 0, 4095));
	addPort(newBoolPortDescriptor("Saw 1"));
	addPort(newBoolPortDescriptor("Tri 1"));
	addPort(newBoolPortDescriptor("Ringmod 1"));
	addPort(newBoolPortDescriptor("Sync 1"));
	addPort(newBoolPortDescriptor("Filter On 1"));
	addPort(newRangePortDescriptor("ENV A 1", 0, 15));
	addPort(newRangePortDescriptor("ENV D 1", 0, 15));
	addPort(newRangePortDescriptor("ENV S 1", 0, 15));
	addPort(newRangePortDescriptor("ENV R 1", 0, 15));

	addPort(newBoolPortDescriptor("Noise 2"));
	addPort(newBoolPortDescriptor("Pulse 2"));
	addPort(newRangePortDescriptor("Pulse Width 2", 0, 4095));
	addPort(newBoolPortDescriptor("Saw 2"));
	addPort(newBoolPortDescriptor("Tri 2"));
	addPort(newBoolPortDescriptor("Ringmod 2"));
	addPort(newBoolPortDescriptor("Sync 2"));
	addPort(newBoolPortDescriptor("Filter On 2"));
	addPort(newRangePortDescriptor("ENV A 2", 0, 15));
	addPort(newRangePortDescriptor("ENV D 2", 0, 15));
	addPort(newRangePortDescriptor("ENV S 2", 0, 15));
	addPort(newRangePortDescriptor("ENV R 2", 0, 15));

	addPort(newBoolPortDescriptor("Noise 3"));
	addPort(newBoolPortDescriptor("Pulse 3"));
	addPort(newRangePortDescriptor("Pulse Width 3", 0, 4095));
	addPort(newBoolPortDescriptor("Saw 3"));
	addPort(newBoolPortDescriptor("Tri 3"));
	addPort(newBoolPortDescriptor("Ringmod 3"));
	addPort(newBoolPortDescriptor("Sync 3"));
	addPort(newBoolPortDescriptor("Filter On 3"));
	addPort(newRangePortDescriptor("ENV A 3", 0, 15));
	addPort(newRangePortDescriptor("ENV D 3", 0, 15));
	addPort(newRangePortDescriptor("ENV S 3", 0, 15));
	addPort(newRangePortDescriptor("ENV R 3", 0, 15));

	port = new LADSPAPortDescriptor
		(
			std::string("Output"),
			
				LADSPAPortDescriptor::Output   |
				LADSPAPortDescriptor::Audio
		);
	addPort(port);

	port = new LADSPAPortDescriptor
		(
			std::string("Gate 1"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			1
		);
	addPort(port);
		port = new LADSPAPortDescriptor
		(
			std::string("Gate 2"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			1
		);
	addPort(port);
		port = new LADSPAPortDescriptor
		(
			std::string("Gate 3"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			1
		);
	addPort(port);
}

SIDDescriptor::~SIDDescriptor()
{
}

LADSPAPluginInstance* SIDDescriptor::create(unsigned long SampleRate)
{
	return new SIDInstance(SampleRate);
}
