/* $Id: SIDInstance.cc,v 1.4 2003/09/14 08:22:08 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "SIDInstance.hh"

/* LADSPA device based on "resid" code, see resid/ directory
   for more information */

#include "resid/sid.h"

#include <cassert>
#include <cstring>
#include <iostream>

static const float clock_freq = 985248;

SIDInstance::SIDInstance(unsigned long sampleRate)
:sampleRate(sampleRate),step(clock_freq/sampleRate)
,port_1_gate_copy(-1),port_2_gate_copy(-1),port_3_gate_copy(-1)
{
	sid.set_chip_model(MOS6581);
	sid.set_sampling_parameters(clock_freq, SAMPLE_FAST, sampleRate);
	sid.enable_filter(true);
	sid.enable_external_filter(true);
	sid.reset();
	std::cerr << "step: " << step << "\n";
}

SIDInstance::~SIDInstance()
{
}

void SIDInstance::connect_port(unsigned long port, LADSPA_Data* dataLocation)
{
	switch (port)
	{
		case  0: port_amp = dataLocation; break;
		case  1: port_1_freq = dataLocation; break;
		case  2: port_2_freq = dataLocation; break;
		case  3: port_3_freq = dataLocation; break;
		case  4: port_mos8580 = dataLocation; break;
		case  5: port_filter = dataLocation; break;
		case  6: port_filter_ext = dataLocation; break;
		case  7: port_filter_lp = dataLocation; break;
		case  8: port_filter_bp = dataLocation; break;
		case  9: port_filter_hp = dataLocation; break;
		case 10: port_filter_cutoff_freq = dataLocation; break;
		case 11: port_filter_resonance = dataLocation; break;
		case 12: port_3off = dataLocation; break;

		case 13: port_1_noise = dataLocation; break;
		case 14: port_1_pulse = dataLocation; break;
		case 15: port_1_pulse_width = dataLocation; break;
		case 16: port_1_saw = dataLocation; break;
		case 17: port_1_tri = dataLocation; break;
		case 18: port_1_ringmod = dataLocation; break;
		case 19: port_1_sync = dataLocation; break;
		case 20: port_1_filter_on = dataLocation; break;
		case 21: port_1_env_a = dataLocation; break;
		case 22: port_1_env_d = dataLocation; break;
		case 23: port_1_env_s = dataLocation; break;
		case 24: port_1_env_r = dataLocation; break;

		case 25: port_2_noise = dataLocation; break;
		case 26: port_2_pulse = dataLocation; break;
		case 27: port_2_pulse_width = dataLocation; break;
		case 28: port_2_saw = dataLocation; break;
		case 29: port_2_tri = dataLocation; break;
		case 30: port_2_ringmod = dataLocation; break;
		case 31: port_2_sync = dataLocation; break;
		case 32: port_2_filter_on = dataLocation; break;
		case 33: port_2_env_a = dataLocation; break;
		case 34: port_2_env_d = dataLocation; break;
		case 35: port_2_env_s = dataLocation; break;
		case 36: port_2_env_r = dataLocation; break;

		case 37: port_3_noise = dataLocation; break;
		case 38: port_3_pulse = dataLocation; break;
		case 39: port_3_pulse_width = dataLocation; break;
		case 40: port_3_saw = dataLocation; break;
		case 41: port_3_tri = dataLocation; break;
		case 42: port_3_ringmod = dataLocation; break;
		case 43: port_3_sync = dataLocation; break;
		case 44: port_3_filter_on = dataLocation; break;
		case 45: port_3_env_a = dataLocation; break;
		case 46: port_3_env_d = dataLocation; break;
		case 47: port_3_env_s = dataLocation; break;
		case 48: port_3_env_r = dataLocation; break;

		case 49: port_output = dataLocation; break;
		case 50: port_1_gate = dataLocation; break;
		case 51: port_2_gate = dataLocation; break;
		case 52: port_3_gate = dataLocation; break;

		default: std::cerr << port << "\n"; assert(false); break;
	}
}

void SIDInstance::activate()
{
}

void SIDInstance::run(unsigned long SampleCount)
{
	assert(SampleCount < 32767);
	LADSPA_Data* buf = port_output;
	unsigned char reg[28];

	//unsigned int Fn1 = (*port_1_freq/clock_freq)*16777216; // TODO port_1_freq[]
	unsigned int Fn1 = (*port_1_freq)/0.0596; // TODO port_1_freq[]
	unsigned int Fn2 = (*port_2_freq)/0.0596; // TODO port_1_freq[]
	unsigned int Fn3 = (*port_3_freq)/0.0596; // TODO port_1_freq[]
	//unsigned int Fn2 = (*port_2_freq/clock_freq)*16777216;
	//unsigned int Fn3 = (*port_3_freq/clock_freq)*16777216;

	if (Fn1_copy != Fn1)
	{
		std::cerr << "Fn1: " << Fn1 << " " << *port_1_freq << "\n";
		Fn1_copy = Fn1;
	}

	unsigned int pw1 = *port_1_pulse_width;
	unsigned int pw2 = *port_2_pulse_width;
	unsigned int pw3 = *port_3_pulse_width;

	reg[ 0] = Fn1 & 255;
	reg[ 7] = Fn2 & 255;
	reg[14] = Fn3 & 255;
	reg[ 1] = (Fn1 & 65280) >> 8;
	reg[ 8] = (Fn2 & 65280) >> 8;
	reg[15] = (Fn3 & 65280) >> 8;
	reg[ 2] = pw1 & 255;
	reg[ 9] = pw2 & 255;
	reg[16] = pw3 & 255;
	reg[ 3] = (pw1 & 3840) >> 8;
	reg[10] = (pw2 & 3840) >> 8;
	reg[17] = (pw3 & 3840) >> 8;
	reg[ 4] =
		((*port_1_noise  )>0?128:0) +
		((*port_1_pulse  )>0?64:0) +
		((*port_1_saw    )>0?32:0) +
		((*port_1_tri    )>0?16:0) +
		((*port_1_ringmod)>0?4:0) +
		((*port_1_sync   )>0?2:0) +
		((*port_1_gate   )>0?1:0);
	reg[11] =
		((*port_2_noise  )>0?128:0) +
		((*port_2_pulse  )>0?64:0) +
		((*port_2_saw    )>0?32:0) +
		((*port_2_tri    )>0?16:0) +
		((*port_2_ringmod)>0?4:0) +
		((*port_2_sync   )>0?2:0) +
		((*port_2_gate   )>0?1:0);
	reg[18] =
		((*port_3_noise  )>0?128:0) +
		((*port_3_pulse  )>0?64:0) +
		((*port_3_saw    )>0?32:0) +
		((*port_3_tri    )>0?16:0) +
		((*port_3_ringmod)>0?4:0) +
		((*port_3_sync   )>0?2:0) +
		((*port_3_gate   )>0?1:0);
	reg[ 5] = ((((unsigned char)*port_1_env_a)&15)<<4) + (((unsigned char)*port_1_env_d)&15);
	reg[12] = ((((unsigned char)*port_2_env_a)&15)<<4) + (((unsigned char)*port_2_env_d)&15);
	reg[19] = ((((unsigned char)*port_3_env_a)&15)<<4) + (((unsigned char)*port_3_env_d)&15);
	reg[ 6] = ((((unsigned char)*port_1_env_s)&15)<<4) + (((unsigned char)*port_1_env_r)&15);
	reg[13] = ((((unsigned char)*port_2_env_s)&15)<<4) + (((unsigned char)*port_2_env_r)&15);
	reg[20] = ((((unsigned char)*port_3_env_s)&15)<<4) + (((unsigned char)*port_3_env_r)&15);
	
	unsigned int fc = *port_filter_cutoff_freq;

	reg[21] = fc & 7;
	reg[22] = (fc & 2040) >> 3;
	reg[23] =
		((((unsigned char)(*port_filter_resonance))&15)<<4) +
		((*port_filter_ext)>0?8:0) +
		((*port_3_filter_on)>0?4:0) +
		((*port_2_filter_on)>0?2:0) +
		((*port_1_filter_on)>0?1:0);
	reg[24] =
		((*port_3off)>0?128:0) +
		((*port_filter_hp)>0?64:0) +
		((*port_filter_bp)>0?32:0) +
		((*port_filter_lp)>0?16:0) +
		((unsigned char)15);
//		(((unsigned char)*port_amp)&15); TODO make amp work again

	for (int i=0; i < 25; i++)
	{
		sid.write(i, reg[i]);
	}

	for (int i=0; i < SampleCount; i++)
	{
		reg[ 4] =
			((*port_1_noise  )>0?128:0) +
			((*port_1_pulse  )>0?64:0) +
			((*port_1_saw    )>0?32:0) +
			((*port_1_tri    )>0?16:0) +
			((*port_1_ringmod)>0?4:0) +
			((*port_1_sync   )>0?2:0) +
			(( port_1_gate[i])>0?1:0);
		reg[11] =
			((*port_2_noise  )>0?128:0) +
			((*port_2_pulse  )>0?64:0) +
			((*port_2_saw    )>0?32:0) +
			((*port_2_tri    )>0?16:0) +
			((*port_2_ringmod)>0?4:0) +
			((*port_2_sync   )>0?2:0) +
			(( port_2_gate[i])>0?1:0);
		reg[18] =
			((*port_3_noise  )>0?128:0) +
			((*port_3_pulse  )>0?64:0) +
			((*port_3_saw    )>0?32:0) +
			((*port_3_tri    )>0?16:0) +
			((*port_3_ringmod)>0?4:0) +
			((*port_3_sync   )>0?2:0) +
			(( port_3_gate[i])>0?1:0);
//		if (port_1_gate[i] != port_1_gate_copy)
//		{
//			std::cerr << "port_1_gate[i]: " << port_1_gate[i] << "\n";
//			port_1_gate_copy = port_1_gate[i];
//		}
//		if (port_2_gate[i] != port_2_gate_copy)
//		{
//			std::cerr << "port_2_gate[i]: " << port_2_gate[i] << "\n";
//			port_2_gate_copy = port_2_gate[i];
//		}
//		if (port_3_gate[i] != port_3_gate_copy)
//		{
//			std::cerr << "port_3_gate[i]: " << port_3_gate[i] << "\n";
//			port_3_gate_copy = port_3_gate[i];
//		}
		sid.write(4, reg[4]);
		sid.write(11, reg[11]);
		sid.write(18, reg[18]);
		sid.clock(step);
		int out = sid.output();
		//if (out > 0) { std::cerr << out << "\n"; }
		buf[i] = out/3276;
	}
}
